#include <stdio.h>
#include <imsls.h>
/* ********************************************************************
 * Two Layer Feed-Forward Network with 4 inputs, all 
 * continuous, and 3 classification categories.
 *     
 * This is a well known database to be found in the pattern
 *     recognition literature.  Fisher's paper is often cited.
 *     The data set contains 3 classes of 50 instances each,
 *     where each class refers to a type of iris plant.  One class is
 *     linearly separable from the other 2; the latter are NOT linearly
 *     separable from each other.
 *
 *  Predicted attribute: class of iris plant.
 *     1=Iris Setosa, 2=Iris Versicolour, and 3=Iris Virginica
 *
 *  Input Attributes (4 Continuous Attributes)
 *     X1: Sepal length, 
 *     X2: Sepal width, 
 *     X3: Petal length, 
 * and X4: Petal width
 * 
 *********************************************************************/
int main(){  
   float *predicted_class_prob, *class_error, *trainStats;
   int i, j;
   int n_patterns    = 150;
   int n_inputs      = 4;   /* all continuous inputs*/
   int n_nominal = 0;       /* no nominal   */
   int n_continuous  = 4;   
   int n_outputs     = 3; 
   int *nominalAtt=NULL;
   int *predicted_class;
   int act_fcn[3] = {1, 1, 1};
   int classification[150];
   float unscaledX[150], scaledX[150];
   float contAtt[4*150];
   float *irisData;
   float mean[4], s[4], center, spread;
   double startTime, endTime;
   char *colLabels[] = {"Pattern", "Class=0", "Class=1", "Class=2"};
   char prtLabel[] = 
       "Predicted_Class  |   P(0)     P(1)     P(2)   | Class_Error";
   char dashes[]  =
       "-------------------------------------------------------------";
   char *filename = "iris_classification.txt";
   /* Declare mlff network structure */
   Imsls_f_NN_Network *network;

   printf("*******************************************************\n");
   printf("* IRIS CLASSIFICATION EXAMPLE                         *\n");
   printf("*******************************************************\n");

	imsls_omp_options(IMSLS_SET_FUNCTIONS_THREAD_SAFE, 1, 0);

   /*
    * irisData[]:  The raw data matrix.  This is a 2-D matrix with 150 
    *              rows and 5 columns. The last 4 columns are the 
    *              continuous input attributes and the 1st column is 
    *              the classification category (1-3).  These data 
    *              contain no nominal input attributes.                                
    */
   irisData = imsls_f_data_sets(3,0);

   /* 
    * Setup the continuous attribute input array, contAtt[], and the 
    * network target classification array, classification[], using the 
    * above raw data matrix.
    */
   for(i=0; i < n_patterns; i++){
      classification[i] = (int)irisData[i*5]-1;
      for(j=1; j<5; j++){
         contAtt[i*4+j-1] = irisData[i*5+j];
      }
   }
   /* Scale continuous input attributes using z-score method */
   for(j=0; j<n_continuous; j++){
      for(i=0; i<n_patterns; i++) 
          unscaledX[i] = contAtt[i*n_continuous+j];
      imsls_f_scale_filter(n_patterns, unscaledX, 2, 
         IMSLS_RETURN_USER, scaledX, 
         IMSLS_RETURN_CENTER_SPREAD, &center, &spread, 0);
      for(i=0; i<n_patterns; i++) 
          contAtt[i*n_continuous+j] = scaledX[i];
      mean[j] = center;
      s[j]    = spread;
   }
   printf("Scale Parameters: \n");
   for(j=0; j<n_continuous; j++){
      printf("Var %d Mean = %f S = %f \n",j+1, mean[j], s[j]);
   } 

   network = imsls_f_mlff_network_init(n_inputs, n_outputs);
   imsls_f_mlff_network(network, IMSLS_CREATE_HIDDEN_LAYER, 2, 
      IMSLS_LINK_ALL, 0);

   /* Note the following statement is for repeatable output */
   imsls_random_seed_set(5555);
   /* Train Classification Network */
   startTime = imsls_ctime();
   trainStats = imsls_f_mlff_classification_trainer (network,
      n_patterns,
      n_nominal,
      n_continuous,
      classification,
      nominalAtt,
      contAtt, 
      IMSLS_PRINT,
      IMSLS_STAGE_I, 20, 150,
      IMSLS_WEIGHT_INITIALIZATION_METHOD, IMSLS_PRINCIPAL_COMPONENTS, 
      IMSLS_MAX_STEP, 1000.0,
      IMSLS_PREDICTED_CLASS, &predicted_class,
      IMSLS_PREDICTED_CLASS_PROB, &predicted_class_prob,
      IMSLS_CLASS_ERROR, &class_error, 
      0);
   endTime = imsls_ctime();
   printf("%s\n", dashes);
   printf("Minimum Cross-Entropy Error: %g\n", trainStats[0]);
   printf("Classification Error Rate:   %f\n", trainStats[5]);
   printf("Execution Time (Sec.):       %f\n\n", (endTime-startTime));

   printf("%s\n",prtLabel);
   printf("%s\n",dashes);
   for(i=0; i<n_patterns; i++){
      printf("       %d   ", predicted_class[i]);
      printf("      | %f %f %f |  %f\n", predicted_class_prob[i*3], 
         predicted_class_prob[i*3+1], predicted_class_prob[i*3+2], 
         class_error[i]);
      if(i==49 || i==99){
          printf("%s\n",prtLabel);
          printf("%s\n",dashes);
      }
   }
      imsls_f_mlff_network_write(network, filename, IMSLS_PRINT, 0);
}
